/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.gef;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.conqat.ide.commons.gef.editpart.EditPartUtils;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.MouseWheelHandler;
import org.eclipse.gef.MouseWheelZoomHandler;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IPostSelectionProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.fortiss.tooling.base.ui.ToolingBaseUIActivator;
import org.fortiss.tooling.base.ui.action.ModelElementsActionBase;
import org.fortiss.tooling.base.ui.dnd.DropTargetListener;
import org.fortiss.tooling.base.ui.editpart.figure.EVisualStyle;
import org.fortiss.tooling.kernel.ui.services.IEditPartFactoryService;

/**
 * GEF editor base implementation for graphical viewer editors.
 * 
 * It provides {@link EditDomain} and thus a {@link CommandStack}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 63FF468A2D042586A22495828A0798EF
 */
public class GraphicalViewerEditorBase<T extends EObject> extends EditorBase<T>
		implements IPostSelectionProvider {

	/** Stores the viewer. */
	protected EditPartViewer viewer = new ScrollingGraphicalViewer();

	/** Actions which need update upon selection changes. */
	private final List<String> selectionActions = new ArrayList<String>();

	/** Actions which need update upon changes of properties. */
	private final List<String> propertyActions = new ArrayList<String>();

	/** Stores the zoom manager. */
	private ZoomManager zoomManager;

	/** Stores the zoom contribution item. */
	private ZoomComboContributionItem zoomContribution;

	/** A container for editor actions */
	private final ActionRegistry actionRegistry = new ActionRegistry();

	/** Selection listener used to update selection actions. */
	private final ISelectionListener selectionListener = new ISelectionListener() {
		@Override
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			updateActions(selectionActions);
		}
	};

	/** {@inheritDoc} */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		super.init(site, input);

		getSite().getWorkbenchWindow().getSelectionService()
				.addSelectionListener(selectionListener);

	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		viewer.createControl(parent);

		getEditDomain().addViewer(viewer);

		viewer.setEditPartFactory(createEditPartFactory());
		viewer.setContents(getViewerContentObject());
		// TODO (FH): add context menu support
		// MenuManager contextMenu = getContextMenuManager();
		// if (contextMenu != null) {
		// viewer.setContextMenu(contextMenu);
		// }

		getSite().setSelectionProvider(this);
		ToolingBaseUIActivator.getDefault().getPreferenceStore()
				.addPropertyChangeListener(propertyChangeListener);

		createActions();
		updateActions(propertyActions);

		createZoomSupport();

		// drag and drop
		viewer.addDropTargetListener(new DropTargetListener(viewer));
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		ToolingBaseUIActivator.getDefault().getPreferenceStore()
				.removePropertyChangeListener(propertyChangeListener);
		super.dispose();
	}

	/**
	 * Returns the root object used as content root. Sub-classes may override to
	 * provide a different content root object. The default implementation uses
	 * {@link EditorBase#getEditedObject()}
	 */
	protected Object getViewerContentObject() {
		return getEditedObject();
	}

	/** Listener used to refresh on editor preference changes. */
	private final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {

		@Override
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getProperty().equals(EVisualStyle.ID)) {
				EditPartUtils.fullRefresh(viewer.getRootEditPart());
			}
		}
	};

	/** Installs zoom suport for this graphical viewer. */
	private void createZoomSupport() {
		RootEditPart rootPart = viewer.getRootEditPart();
		if (rootPart instanceof ScalableFreeformRootEditPart) {
			zoomManager = ((ScalableFreeformRootEditPart) rootPart)
					.getZoomManager();
		} else if (viewer.getRootEditPart() instanceof ScalableRootEditPart) {
			zoomManager = ((ScalableRootEditPart) rootPart).getZoomManager();
		}

		if (zoomManager != null) {
			zoomManager.setZoomLevelContributions(getZoomLevelContributions());
			registerZoomAction();
		}
		zoomContribution = new ZoomComboContributionItem(getSite().getPage());
	}

	/** Registers and returns the id of the given action. */
	protected final String registerAction(IAction action) {
		actionRegistry.registerAction(action);
		return action.getId();
	}

	/** Returns the zoom manager */
	protected ZoomManager getZoomManager() {
		return zoomManager;
	}

	/** Registers zoom actions. */
	private void registerZoomAction() {
		registerAction(new ZoomInAction(zoomManager));
		registerAction(new ZoomOutAction(zoomManager));

		viewer.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.MOD1),
				MouseWheelZoomHandler.SINGLETON);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		// This is called, e.g., from the PrintAction
		if (adapter == GraphicalViewer.class) {
			return viewer;
		}
		// This adapter is required for the ZoomComboContributionItem
		if (adapter == ZoomManager.class) {
			return getZoomManager();
		}
		return super.getAdapter(adapter);
	}

	/**
	 * Creates the {@link EditPartFactory} which is used for this editor. The
	 * default implementation uses the {@link EditPartFactory}.
	 */
	protected EditPartFactory createEditPartFactory() {
		return IEditPartFactoryService.INSTANCE;
	}

	/**
	 * Returns the zoom level contributions. The default implementation allows
	 * {@link ZoomManager#FIT_ALL}, {@link ZoomManager#FIT_WIDTH}, and
	 * {@link ZoomManager#FIT_HEIGHT}.
	 */
	protected List<String> getZoomLevelContributions() {
		return defaultZoomLevelContribution;
	}

	/**
	 * Default zoom level contributions.
	 */
	private static final List<String> defaultZoomLevelContribution = Arrays
			.asList(new String[] { ZoomManager.FIT_ALL, ZoomManager.FIT_WIDTH,
					ZoomManager.FIT_HEIGHT });

	/** Registers global actions (including zoom action). */
	public void registerGlobalActions(IActionBars bars) {
		bars.getToolBarManager().add(zoomContribution);
	}

	/** Unregisters global actions (including zoom action). */
	public void unregisterGlobalActions(IActionBars bars) {
		bars.getToolBarManager().remove(zoomContribution);
	}

	/** Creates actions for this editor. */
	private void createActions() {
		// registerSelectAllAction();
		// registerPrintAction();
		// registerEditActions();
		// registerOtherActions();
	}

	/** Updates a list of actions. */
	private void updateActions(List<String> actionIds) {
		final ActionRegistry registry = actionRegistry;
		for (final String id : actionIds) {
			final IAction action = registry.getAction(id);
			if (action instanceof UpdateAction) {
				((UpdateAction) action).update();
			} else if (action instanceof ModelElementsActionBase) {
				((ModelElementsActionBase) action).update();
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	// The following methods are necessary in order to avoid Eclipse Helios SR2
	// bug #339360

	/** {@inheritDoc} */
	@Override
	public void addPostSelectionChangedListener(
			ISelectionChangedListener listener) {
		viewer.addSelectionChangedListener(listener);
	}

	/** {@inheritDoc} */
	@Override
	public void removePostSelectionChangedListener(
			ISelectionChangedListener listener) {
		viewer.removeSelectionChangedListener(listener);
	}

	/** {@inheritDoc} */
	@Override
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		viewer.addSelectionChangedListener(listener);
	}

	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return viewer.getSelection();
	}

	/** {@inheritDoc} */
	@Override
	public void removeSelectionChangedListener(
			ISelectionChangedListener listener) {
		viewer.removeSelectionChangedListener(listener);
	}

	/** {@inheritDoc} */
	@Override
	public void setSelection(ISelection selection) {
		viewer.setSelection(selection);
	}
}
